// Copyright Epic Games, Inc. All Rights Reserved.

#include "EndlessVendettaCharacter.h"
#include "Animation/AnimInstance.h"
#include "Camera/CameraComponent.h"
#include "Components/CapsuleComponent.h"
#include "EnhancedInputComponent.h"
#include "EnhancedInputSubsystems.h"


//////////////////////////////////////////////////////////////////////////
// AEndlessVendettaCharacter

AEndlessVendettaCharacter::AEndlessVendettaCharacter()
{
	// Character doesnt have a rifle at start
	bHasRifle = false;

	// Set size for collision capsule
	GetCapsuleComponent()->InitCapsuleSize(55.f, 96.0f);

	// Create a CameraComponent	
	FirstPersonCameraComponent = CreateDefaultSubobject<UCameraComponent>(TEXT("FirstPersonCamera"));
	FirstPersonCameraComponent->SetupAttachment(GetCapsuleComponent());
	FirstPersonCameraComponent->SetRelativeLocation(FVector(-10.f, 0.f, 60.f)); // Position the camera
	FirstPersonCameraComponent->bUsePawnControlRotation = true;

	// Create a mesh component that will be used when being viewed from a '1st person' view (when controlling this pawn)
	Mesh1P = CreateDefaultSubobject<USkeletalMeshComponent>(TEXT("CharacterMesh1P"));
	Mesh1P->SetOnlyOwnerSee(true);
	Mesh1P->SetupAttachment(FirstPersonCameraComponent);
	Mesh1P->bCastDynamicShadow = false;
	Mesh1P->CastShadow = false;
	//Mesh1P->SetRelativeRotation(FRotator(0.9f, -19.19f, 5.2f));
	Mesh1P->SetRelativeLocation(FVector(-30.f, 0.f, -150.f));

}

void AEndlessVendettaCharacter::BeginPlay()
{
	// Call the base class  
	Super::BeginPlay();

	//Add Input Mapping Context
	if (APlayerController* PlayerController = Cast<APlayerController>(Controller))
	{
		if (UEnhancedInputLocalPlayerSubsystem* Subsystem = ULocalPlayer::GetSubsystem<UEnhancedInputLocalPlayerSubsystem>(PlayerController->GetLocalPlayer()))
		{
			Subsystem->AddMappingContext(DefaultMappingContext, 0);
		}
	}
	AActor* GadgetManagerActor = GetWorld()->SpawnActor<AActor>(GadgetManagerClass, GetActorLocation(), GetActorRotation());
	GadgetManager = Cast<AGadgetManager>(GadgetManagerActor);
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
	GadgetManagerActor->AttachToComponent(GetRootComponent(), AttachmentRules);
	for (UActorComponent* PlayersCamera : GetComponentsByTag(UCameraComponent::StaticClass(), FName("PlayersCamera")))
	{
		UE_LOG(LogTemp, Warning, TEXT("An actor component with PlayersCamera tag has been found"));
		GadgetManager->SpawnGadgets(Cast<USceneComponent>(PlayersCamera));
		break;
	}
	
}

//////////////////////////////////////////////////////////////////////////// Input

void AEndlessVendettaCharacter::SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent)
{
	// Set up action bindings
	if (UEnhancedInputComponent* EnhancedInputComponent = CastChecked<UEnhancedInputComponent>(PlayerInputComponent))
	{
		//Jumping
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Triggered, this, &ACharacter::Jump);
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Completed, this, &ACharacter::StopJumping);

		//Moving
		EnhancedInputComponent->BindAction(MoveAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::Move);

		//Looking
		EnhancedInputComponent->BindAction(LookAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::Look);

		// Gadget Toggling
		EnhancedInputComponent->BindAction(ReconAction, ETriggerEvent::Started, this, &AEndlessVendettaCharacter::ToggleRecon);
		EnhancedInputComponent->BindAction(CombatAction, ETriggerEvent::Started, this, &AEndlessVendettaCharacter::ToggleCombat);
		
		//Weapon Switching
		EnhancedInputComponent->BindAction(EquipPrimaryWeapon, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::EquipPrimary);
		EnhancedInputComponent->BindAction(EquipSecondaryWeapon, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::EquipSecondary);

		//Weapon Shooting
		EnhancedInputComponent->BindAction(TapShootAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::TapFireCaller);
	}
}

void AEndlessVendettaCharacter::ToggleRecon()
{
	if (IsValid(PrimaryWeapon)) EquipPrimary();
	if (IsValid(SecondaryWeapon)) EquipSecondary();

	if (GadgetManager->IsReconEquipped())
	{
		GadgetManager->TryToUnequipRecon();
		return;
	}

	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat())
	{
		// Do nothing if combat is equipped and can't be unequipped at this moment
		return;
	}

	GadgetManager->EquipRecon();
}

void AEndlessVendettaCharacter::ToggleCombat()
{
	if (IsValid(PrimaryWeapon)) EquipPrimary();
	if (IsValid(SecondaryWeapon)) EquipSecondary();

	if (GadgetManager->IsCombatEquipped())
	{
		GadgetManager->TryToUnequipCombat();
		return;
	}

	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon())
	{
		// Do nothing if recon is equipped and can't be unequipped at the moment
		return;
	}

	GadgetManager->EquipCombat();
}


//When 1 is pressed it calls EquipPrimary
void AEndlessVendettaCharacter::EquipPrimary()
{
	if (IsValid(PrimaryWeapon))
	{
		PrimaryWeapon->Destroy();
		PrimaryWeapon = nullptr;
		bHasRifle = false;
		return;
	}

	if (IsValid(SecondaryWeapon)) EquipSecondary();
	
	// Marcel I respect you, like you and what not, please for the love of god don't touch these two guard clauses
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon()) return;/////////////////////////
	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat()) return;///////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	FActorSpawnParameters spawnParams;
	spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);

	bHasRifle = true;
	//Creating a new actor object called PrimaryWeapon that is based off primaryweaponClass
	AActor* PrimaryWeaponActor = GetWorld()->SpawnActor<AActor>(PrimaryWeaponClass, spawnParams);
	PrimaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
	//Changing PrimaryWeaponActor to ABaseWeaponClass type instead of actor and storing it into PrimaryWeapon which is a ABaseClass Object
	//We do this because we need to check if PrimaryWeapon is equipped and we want primaryweapon to be ABaseWeapon type and not a generic AActor
	PrimaryWeapon = Cast<ABaseWeaponClass>(PrimaryWeaponActor);
	
	//If primary weapon is not there but secondary weapon is it will call equipSecondary.
	//EquipSecondary checks and sees that secondary is there so it will call to destroy itself
	//Code goes back and sees primary weapon is not there anymore and spawns it in.
	//Same thing for the EquipSecondary()

	
	// if (!IsValid(PrimaryWeapon))
	// {
	// 	if (IsValid(SecondaryWeapon))  EquipSecondary();
	//
	// 	UE_LOG(LogTemp, Display, TEXT("Primary equipped"));
		// bHasRifle = true;
		// FActorSpawnParameters spawnParams;
		// spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
		// FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
		// //Creating a new actor object called PrimaryWeapon that is based off primaryweaponClass
		// AActor* PrimaryWeaponActor = GetWorld()->SpawnActor<AActor>(PrimaryWeaponClass, spawnParams);
		// PrimaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
		// //Changing PrimaryWeaponActor to ABaseWeaponClass type instead of actor and storing it into PrimaryWeapon which is a ABaseClass Object
		// //We do this because we need to check if PrimaryWeapon is equipped and we want primaryweapon to be ABaseWeapon type and not a generic AActor
		// PrimaryWeapon = Cast<ABaseWeaponClass>(PrimaryWeaponActor);
	// 	
	// }
	// else if (IsValid(PrimaryWeapon))
	// {
	// 	PrimaryWeapon->Destroy();
	// 	bHasRifle = false;
	// }
}

void AEndlessVendettaCharacter::EquipSecondary()
{
	if (IsValid(SecondaryWeapon))
 	{
 		SecondaryWeapon->Destroy();
		SecondaryWeapon = nullptr;
 		bHasRifle = false;
 		return;
 	}

	if (IsValid(PrimaryWeapon)) EquipPrimary();
	
	// Marcel I respect you, like you and what not, please for the love of god don't touch these two guard clauses
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon()) return;/////////////////////////
	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat()) return;///////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	FActorSpawnParameters spawnParams;
	spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
	
	bHasRifle = true;
	AActor* SecondaryWeaponActor = GetWorld()->SpawnActor<AActor>(SecondaryWeaponClass, spawnParams);
	SecondaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
	SecondaryWeapon = Cast<ABaseWeaponClass>(SecondaryWeaponActor);
	
	// if (!IsValid(SecondaryWeapon))
	// {
	// 	if (IsValid(PrimaryWeapon))  EquipPrimary();
	//
	// 	UE_LOG(LogTemp, Display, TEXT("Secondary equipped"));
		// bHasRifle = true;
		// FActorSpawnParameters spawnParams;
		// spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
		// FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
		// AActor* SecondaryWeaponActor = GetWorld()->SpawnActor<AActor>(SecondaryWeaponClass, spawnParams);
		// SecondaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
		// SecondaryWeapon = Cast<ABaseWeaponClass>(SecondaryWeaponActor);
	// }
	// else if (IsValid(SecondaryWeapon))
	// {
	// 	SecondaryWeapon->Destroy();
	// 	bHasRifle = false;
	// }
	
}

//Calls the fire function in the baseWeaponClass
void AEndlessVendettaCharacter::TapFireCaller()
{
	UE_LOG(LogTemp, Warning, TEXT("Tap Fire"));
	// if (IsValid(PrimaryWeapon) || IsValid(SecondaryWeapon))
	// {
	// 	PrimaryWeaponClass.GetDefaultObject()->TapFire();
	// }
}

//POTENTIAL PICKUP SYSTEM NEEDS CHANGING BUT YES

//Called from Player BluePrints
/*void AEndlessVendettaCharacter::WeaponPickUpSystem(AActor* PickedUpWeapon)
{
	ABaseWeaponClass* WeaponInWorld = Cast<ABaseWeaponClass>(PickedUpWeapon);
	if(WeaponInWorld->IsA(ABaseWeaponClass::StaticClass()))
	{
		if(PrimaryWeapon)
		{
			UE_LOG(LogTemp, Display, TEXT("Primary Weapon Picked up"));
			bHasRifle = true;
			FVector loc = GetActorLocation() + FVector(-50, 0, 0);
			FRotator rot = GetActorRotation();
			FActorSpawnParameters spawnParams;
			spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
			FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
			//WeaponInWorld = Cast<ABaseWeaponClass>(GetWorld()->SpawnActor<AActor>(PrimaryWeapon, loc, rot, spawnParams));
			WeaponInWorld->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
			WeaponInWorld->Destroy();

			//for some reason the spawning of the weapon is broken to the players hand so need to fix this.

			// potentially add a varibable to check if its a secondary or priamry weapon being picked up
			// add this in the baseweaponclass variable
		}
	}
	if(WeaponInWorld->IsA(ABaseWeaponClass::StaticClass()))
	{
		if(SecondaryWeapon)
		{
			UE_LOG(LogTemp, Display, TEXT("Secondary Weapon Picked up"));

		}
	}
}*/


void AEndlessVendettaCharacter::Move(const FInputActionValue& Value)
{
	// input is a Vector2D
	FVector2D MovementVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add movement 
		AddMovementInput(GetActorForwardVector(), MovementVector.Y);
		AddMovementInput(GetActorRightVector(), MovementVector.X);
	}
}

void AEndlessVendettaCharacter::Look(const FInputActionValue& Value)
{
	// input is a Vector2D
	FVector2D LookAxisVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add yaw and pitch input to controller
		AddControllerYawInput(LookAxisVector.X);
		AddControllerPitchInput(LookAxisVector.Y);
	}
}


void AEndlessVendettaCharacter::SetHasRifle(bool bNewHasRifle)
{
	bHasRifle = bNewHasRifle;
}

bool AEndlessVendettaCharacter::GetHasRifle()
{
	return bHasRifle;
}